import { useMemo, useState, memo, useReducer, useEffect, useId } from "react";
import IcBaselineDeleteForever from "@/components/icons/IcBaselineDeleteForever";
import IconamoonShoppingBagBold from "@/components/icons/IconamoonShoppingBagBold";
import Select from 'react-select';
import Timer from "easytimer.js";
import api from "@/lib/api";
import { toast } from "react-toastify";
import type {BaseSyntheticEvent} from 'react';

type BagsReducerState = (string|number)[];
type BagsReducerAction = ['add_bag']|['set_weight', number, string]|['delete_bag', number];
type CustomerNameReactSelect = {value: string, label: string};
interface BagsProps {
	bags: BagsReducerState
	dispatchBags: (action: BagsReducerAction)=>void
	total_weight: number
	weight_error: boolean
}

function bagsReducer(state: BagsReducerState, action: BagsReducerAction){
	let bags = [...state];
	if(action[0]==='set_weight') bags[action[1]] = action[2];
	else if(action[0]==='add_bag') bags.push('');
	else if(action[0]==='delete_bag'){
		delete bags[action[1]];
		bags = bags.filter(v=>v!==undefined);
	}
	return bags;
}

const Bags = memo(function Bags({bags, dispatchBags, total_weight, weight_error}: BagsProps){
	return <>
		{
			bags.map((v, k)=>(
				<div key={k} className={`grid gap-x-2 items-center mt-4 ${bags.length>1 ? 'grid-cols-[1fr_auto]' : ''}`}>
					<div className="grid grid-cols-[1fr_auto] gap-x-2 justify-around items-center border border-base-content/20 rounded-[0.3125rem]">
						<label className="input-float">
							<input type="number" value={v} onInput={(event:BaseSyntheticEvent<InputEvent>)=>{dispatchBags(['set_weight', k, event.target.value])}} placeholder="" min='0' className="!border-0" />
							<p>Laundry bag {k+1} (lb)</p>
						</label>
					</div>
					{ bags.length>1 && <button onClick={()=>{dispatchBags(['delete_bag', k])}} className="btn btn-square btn-error btn-outline fill-error hover:fill-white"><div className="w-7 h-7"><IcBaselineDeleteForever /></div></button> }
				</div>
			))
		}
		{ weight_error && <p className="text-error">* Invalid weight</p> }
		<div className="flex items-center justify-between">
			<button onClick={()=>{dispatchBags(['add_bag'])}} className="btn btn-sm btn-primary mt-2 stroke-gray-200 text-gray-200 gap-x-1">
				<div className="w-5 h-5"><IconamoonShoppingBagBold /></div>
				<p>Add bag</p>
			</button>
			<p>Total weight: <span className="text-primary font-bold">{total_weight}</span>lb</p>
		</div>
	</>
});

export default memo(function ManualJob({setTab}: {setTab: (value: 'manual_job'|'completed_jobs')=>void}){
	const [step, setStep] = useState<'info'|'stopwatch'>('info');
	const customer_name_select_id = useId();
	const [customer_name_options, setCustomerNameOptions] = useState<CustomerNameReactSelect[]>([]);
	const [customer_name, setCustomerName] = useState<CustomerNameReactSelect>();
	const [customer_name_error, setCustomerNameError] = useState(false);
	const [bags, dispatchBags] = useReducer(bagsReducer, ['']);
	const [counter, setCounter] = useState({ seconds: 0, minutes: 0, hours: 0, is_paused: false });
	const [weight_error, setWeightError] = useState(false);
	const total_weight = useMemo(()=>Number(bags.reduce((a: number, v)=>v ? Number(v)+a : a, 0).toFixed(2)), [bags]);

	const timer = useMemo(()=>{
		const timer = new Timer();
		timer.addEventListener('secondsUpdated', ()=>{
			const t = timer.getTimeValues();
			setCounter({
				seconds: t.seconds,
				minutes: t.minutes,
				hours: t.hours,
				is_paused: false
			});
		});
		return timer;
	}, []);

	function startTimer(){
		setCustomerNameError(!customer_name);
		if(!customer_name) return;
		timer.start();
		setStep('stopwatch');
	}

	function toggleResumePause(){
		counter.is_paused ? timer.start() : timer.pause();
		setCounter({...counter, is_paused: !counter.is_paused});
	}

	async function complete(){
		const is_weight_error = total_weight<=0;
		setWeightError(is_weight_error);
		if(is_weight_error || !customer_name) return;
		const seconds = timer.getTotalTimeValues().seconds;
		timer.stop();
		await api.post('/admin/jobs', {
			cid: customer_name.value,
			bags: bags.map(v=>Number(v)),
			stopwatch: seconds
		})
		toast.success('Job has been completed successfully');
		setTab('completed_jobs');
	}

	useEffect(()=>{
		(async ()=>{
			const {data}: {data: {id: string, name: string}[]} = await api.get('/admin/customers');
			setCustomerNameOptions(data.map(v=>({value: v.id, label: v.name})));
		})()
	}, []);

	return <>
		{
			step==='info' &&
			<div>
				<Select options={customer_name_options} instanceId={customer_name_select_id} value={customer_name} onChange={(single_value)=>{setCustomerName(single_value as CustomerNameReactSelect); setCustomerNameError(false);}} placeholder='Customer name' />
				{ customer_name_error && <p className="text-error">* Customer name is required</p> }
				<Bags bags={bags} dispatchBags={dispatchBags} total_weight={total_weight} weight_error={weight_error} />
				<button className="btn mx-auto block mt-4 btn-success btn-outline" onClick={startTimer} >Start stopwatch</button>
			</div>
		}
		{
			step==='stopwatch' &&
			<div>
				<div>
					<p>Customer name: {customer_name?.label}</p>
					<Bags bags={bags} dispatchBags={dispatchBags} total_weight={total_weight} weight_error={weight_error} />
				</div>
				<div className="grid grid-flow-col gap-5 text-center auto-cols-max justify-center mt-4">
					<div className="flex flex-col p-2 bg-neutral rounded-box text-neutral-content">
						<span className="countdown font-mono text-5xl">
							<span style={{"--value": counter.hours} as unknown as any}></span>
						</span>
						hour{counter.hours>1 && 's'}
					</div> 
					<div className="flex flex-col p-2 bg-neutral rounded-box text-neutral-content">
						<span className="countdown font-mono text-5xl">
							<span style={{"--value": counter.minutes} as unknown as any}></span>
						</span>
						min
					</div> 
					<div className="flex flex-col p-2 bg-neutral rounded-box text-neutral-content">
						<span className="countdown font-mono text-5xl">
							<span style={{"--value": counter.seconds} as unknown as any}></span>
						</span>
						sec
					</div>
				</div>
				<div className="mt-3 flex justify-center gap-x-20">
					<button onClick={toggleResumePause} className={`btn btn-outline ${counter.is_paused ? 'btn-primary' : 'btn-error text-white'}`}>{counter.is_paused ? 'Resume' : 'Pause'}</button>
					<button onClick={complete} className="btn btn-success btn-outline text-white">Complete</button>
				</div>
			</div>
		}
	</>
})
